# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ConfigParser


def read(config_name):
    """Given a filename, read the configuration."""
    cfg = ConfigParser.SafeConfigParser()
    cfg.read(config_name)
    if not cfg.has_section('pinboard'):
        cfg.add_section('pinboard')
    if not cfg.has_option('pinboard', 'token'):
        cfg.set('pinboard', 'token', '')
    if not cfg.has_section('history'):
        cfg.add_section('history')
    if not cfg.has_option('history', 'since'):
        cfg.set('history', 'since', '')
    return cfg


def save(cfg, config_name, since=None):
    """Write a new config file, including an update to 'since' if needed."""
    if since:
        cfg.set('history', 'since', str(since))
    with open(config_name, 'w') as f:
        cfg.write(f)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import BaseHTTPServer
import logging
import threading
import urllib
import webbrowser

import requests


LOG = logging.getLogger(__name__)


class AuthError(Exception):
    def __init__(self, status_code, error_msg):
        super(AuthError, self).__init__('Authentication failed: %s: %s'
                                        % (status_code, error_msg))
        self.status_code = status_code
        self.error_msg = error_msg


class AuthResponseHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_GET(self):
        self.send_response(200)
        self.end_headers()
        self.wfile.write('''
        <html><head><title>pocket2pinboard</title></head>
        <body>
        <h1>pocket2pinboard</h1>
        <p>You may close this window now.</p>
        <script type="text/javascript">
        function closeMe() {
            window.close();
        }
        window.onload = closeMe;
        </script></body>
        ''')
        self.server.responded.set()
        return

    def log_request(self, code, size=0):
        # Do not log requests
        return


def _start_auth_server():
    server = BaseHTTPServer.HTTPServer(('127.0.0.1', 0),
                                       AuthResponseHandler)
    ip, port = server.server_address
    url = 'http://%s:%s' % (ip, port)
    responded = threading.Event()
    server.responded = responded
    t = threading.Thread(name='auth-response-server',
                         target=server.serve_forever,
                         )
    t.setDaemon(True)
    t.start()

    def _wait_for_auth_response():
        LOG.info('waiting for application authentication')
        responded.wait()
        server.shutdown()

    return (url, _wait_for_auth_response)


def authenticate(consumer_key):
    # Start the HTTP server in a separate thread here so it is running
    # when we ask for the request token. If the user has already logged in
    # and permitted access to the app the redirect is called right away
    # when the browser opens the auth_url.
    redirect_url, waiter = _start_auth_server()

    # Get the request code.
    headers = {
        'X-Accept': 'application/json',
    }
    payload = {
        'consumer_key': consumer_key,
        'redirect_uri': redirect_url,
    }
    request_token_url = 'https://getpocket.com/v3/oauth/request'
    response = requests.post(request_token_url, data=payload, headers=headers)
    request_code = response.json()['code']

    # Ask the user for permission for the app to access their account.
    auth_url_base = 'https://getpocket.com/auth/authorize'
    auth_url_params = urllib.urlencode({
        'request_token': request_code,
        'redirect_uri': redirect_url,
    })
    auth_url = auth_url_base + '?' + auth_url_params
    webbrowser.open(auth_url)

    # Wait for the redirect URL to be accessed in the browser.
    waiter()

    # Check if we were given permission to access the account.
    authorize_url = 'https://getpocket.com/v3/oauth/authorize'
    payload = {
        'consumer_key': consumer_key,
        'code': request_code,
    }
    response = requests.post(authorize_url, data=payload, headers=headers)
    if response.status_code == 200:
        return response.json()['access_token']
    raise AuthError(response.status_code, response.text)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import datetime
import logging

import requests


LOG = logging.getLogger(__name__)


_url = 'https://getpocket.com/v3/get'
_headers = {
    'X-Accept': 'application/json',
}


PocketItem = collections.namedtuple(
    'PocketItem',
    ['url', 'title', 'excerpt', 'time_updated', 'tags']
)


def make_pocket_item(i):
    tags = i.get('tags', {}).keys()
    title = (i.get('resolved_title') or u'No title').encode('utf-8')
    time_updated = datetime.datetime.fromtimestamp(
        float(i['time_updated'])
    )
    excerpt = i.get('excerpt', u'').encode('utf-8')
    return PocketItem(
        url=i['resolved_url'],
        title=title,
        excerpt=excerpt,
        time_updated=time_updated,
        tags=tags,
    )


def get_items(consumer_key, access_token, since):
    payload = {
        'consumer_key': consumer_key,
        'access_token': access_token,
        'state': 'all',
        'detailType': 'complete',
    }
    if since:
        payload['since'] = since
    response = requests.post(_url, data=payload, headers=_headers)
    if response.status_code == 200:
        data = response.json()
        new_since = data['since']
        items = data['list']
        # If the list is empty, we get a list. If it has values, it is a
        # dictionary mapping ids to contents. We want to iterate over all
        # of them, so just make a list of the values.
        if isinstance(items, dict):
            items = list(items.values())
        return ((make_pocket_item(i) for i in items), new_since)
    raise RuntimeError('could not retrieve: %s: %s' %
                       (response.status_code, response.text))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

LOG = logging.getLogger(__name__)


def update(pinboard_client, items):
    for i in items:
        if not i.tags:
            # Skip anything that isn't tagged.
            continue
        LOG.info('%s - %s: %s' % (i.time_updated.date(), i.title, i.tags))
        LOG.debug('%r', i)
        pinboard_client.posts.add(
            url=i.url,
            description=i.title,
            extended=i.excerpt,
            tags=u', '.join(i.tags),
            date=str(i.time_updated.date()),
        )
        LOG.debug('')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import argparse
import datetime
import logging
import os
import sys

import pinboard

import pkg_resources

from pocket2pinboard import auth
from pocket2pinboard import bookmarks
from pocket2pinboard import config
from pocket2pinboard import keys
from pocket2pinboard import retrieve

LOG = logging.getLogger(__name__)


def main():
    # Load the configuration file and determine default values for a
    # few parameters.
    config_name = os.path.expanduser('~/.pocket2pinboardrc')
    cfg = config.read(config_name)
    since = cfg.get('history', 'since')
    pinboard_token = cfg.get('pinboard', 'token')
    if not pinboard_token:
        # Save the skeleton file to make editing it easier later.
        config.save(cfg, config_name)

    # Handle command line arguments.
    parser = argparse.ArgumentParser()
    dist = pkg_resources.get_distribution('pocket2pinboard')
    parser.add_argument(
        '--version',
        action='version',
        version='%(prog)s {0}'.format(dist.version),
    )
    parser.add_argument('-v',
                        dest='verbose',
                        action='store_true',
                        default=False,
                        help='provide verbose output',
                        )
    parser.add_argument('--all',
                        action='store_true',
                        default=False,
                        help='process all pocket articles',
                        )
    parser.add_argument(
        '--pinboard-token', '-t',
        default=pinboard_token,
        help='pinboard.in API token',
    )
    args = parser.parse_args()

    # Make sure we know how to talk to pinboard.in.
    if not args.pinboard_token:
        parser.error(
            'Please update the pinboard token in %s or provide one via -t'
            % config_name)

    # Set up logging for use as output channel.
    level = logging.DEBUG if args.verbose else logging.INFO
    logging.basicConfig(
        level=level,
        format='%(message)s',
    )
    if not args.verbose:
        requests_logger = logging.getLogger('requests')
        requests_logger.setLevel(logging.WARNING)

    try:
        # Connect to both services early to find authentication
        # issues.
        access_token = auth.authenticate(keys.consumer_key)
        pinboard_client = pinboard.Pinboard(args.pinboard_token)

        # Get a list of the pocket items we are going to process.
        if args.all:
            since = None
        if since:
            LOG.info('loading pocket items since %s',
                     datetime.datetime.fromtimestamp(float(since)))
        else:
            LOG.info('fetching all pocket items')

        items, new_since = retrieve.get_items(
            keys.consumer_key,
            access_token,
            since,
        )

        # Send the pocket items to pinboard.
        bookmarks.update(pinboard_client, items)

        # Remember the new value for 'since'.
        config.save(cfg, config_name, new_since)
    except Exception as e:
        if args.verbose:
            raise
        parser.error(e)
        sys.exit(1)
